#!/usr/bin/env python3
 
import datetime
import os
import sys

from Octopus.OctopusCore.configuration import Configuration


class Globals:
    # Global class hold a global variables for reporting and environment managmaent
    class Report:
        # Reporter class is defining a SET OF VARIABLES that eill be used to handle custom html report
        CycleTotalTestCases=0
        ReportFolder=""                
        ResultFolder=""
        ScreenShootFolder=""
        ScreenShootHTMLPath=""
        FinalResults=""

        StartTime=""
        EndTime=""
        Duration=""

        TestCaseIterationsCount=0

        PassCount=0
        FailCount=0

        TotalPassCount=0
        TotalFailCount=0

        PassSteps=0
        FailSteps=0

        TotalFailSteps=0
        TotalPassSteps=0
        
        TotalTestCaseCount=0

        ScreenCaptureCount=0

        IterationStartTime=""
        IterationEndTime=""

        ReportFileName=""
        ReportHTML=""

        TestCaseStatus=False
        TestStatus=False

        @staticmethod
        def initializeRepoeterGlobals(envVars):
            try:
                Globals.Report.CycleTotalTestCases=0
                Globals.Report.ReportFolder=os.getcwd()+Configuration.appSettings["ReportPath"]            
                Globals.Report.SnapshotCaptureLevel=""
                Globals.Report.ResultFolder=Globals.Report.ReportFolder+"\\Automation_Result_" + datetime.datetime.today().strftime("%Y%m%d%H%M%S")
                Globals.Report.ScreenShootFolder=Globals.Report.ResultFolder+"\\ScreenShots_" + (envVars["ProjectName"] if "ProjectName" in envVars else "Project")
                Globals.Report.ScreenShootHTMLPath = "ScreenShots_" + (envVars["ProjectName"] if "ProjectName" in envVars else "Project")
                Globals.Report.ReportFileName=Globals.Report.ResultFolder+"\\Automation_Report_" + (envVars["ProjectName"] if "ProjectName" in envVars else "Project") +".html"
                Globals.Report.FinalResults=""
                Globals.Report.StartTime=""
                Globals.Report.EndTime=""
                Globals.Report.TestCaseIterationsCount=0
                Globals.Report.PassCount=0
                Globals.Report.FailCount=0
                Globals.Report.TotalPassCount=0
                Globals.Report.TotalFailSteps=0
                Globals.Report.TotalFailCount=0
                Globals.Report.TotalTestCaseCount=0
                Globals.Report.ScreenCaptureCount=0
                Globals.Report.IterationStartTime=""            
                Globals.Report.TestCaseStatus=False
                if not os.path.exists(Globals.Report.ResultFolder):
                    os.makedirs(Globals.Report.ResultFolder)
                if not os.path.exists(Globals.Report.ScreenShootFolder):
                    os.makedirs(Globals.Report.ScreenShootFolder)
                if not os.path.exists(Globals.Report.ReportFileName):
                    open(Globals.Report.ReportFileName, 'a+').close() 
            except:
                print(f'Failed to initialize Report global variables for error : {sys.exc_info()[1]}')

    class Test:
        # Test class will define a set of variables to used for managing the test
        BrowserTimeOut=0
        isFirstLogin=False
        Browser=None
        RunEnvironment=""
        URL=""
        BrowserName=""
        RunType=""
        ProjectName=""
        CustomReportEnable=False
        LogedInUserName=""
        IsLogedIn=False
        FrameworkPath=""
        MobileServerURL=""
        isTestParallel=False
        MobileAPP=None
        AndriodDeviceDesc=None
        IOSDeviceDesc=None
        TestStatus=0

        @staticmethod
        def initializeTestGlobals(envVars):
            try:
                Globals.Test.BrowserTimeOut=0
                Globals.Test.isFirstLogin=False
                Globals.Test.Browser=None
                Globals.Test.RunEnvironment=envVars["RunEnvironment"] if "RunEnvironment" in envVars else "WEB"
                Globals.Test.URL=envVars["URL"] if "URL" in envVars else "http://localhost:8080"
                Globals.Test.BrowserName=envVars["Explorer"] if "Explorer" in envVars else "CHROME"
                Globals.Test.RunType=envVars["RunType"] if "RunType" in envVars else "DEBUG"
                Globals.Test.ProjectName=envVars["ProjectName"] if "ProjectName" in envVars else "project"
                Globals.Test.CustomReportEnable=False
                Globals.Test.LogedInUserName=""
                Globals.Test.IsLogedIn=False
                Globals.Test.FrameworkPath=os.getcwd()
                Globals.Test.MobileServerURL=""
                Globals.Test.isTestParallel=False
                Globals.Test.MobileAPP=None
                Globals.Test.AndriodDeviceDesc=None
                Globals.Test.IOSDeviceDesc=None
                Globals.Test.TestStatus=0
            except:
                print(f'Failed to initialize Test global variables for error : {sys.exc_info()[1]}')
    
    # storage for the instance reference
    __Report = None
    __Test = None

    def __init__(self,subCls):
        """ Create singleton instance """
        # Check whether we already have an instance
        if type(subCls) is Globals.Report:
            if Globals.__Report is None:
                # Create and remember instance
                Globals.__Report = Globals.Report()
                # Store instance reference as the only member in the handle
                self.__dict__['_Globals__Report'] = Globals.__Report
        elif type(subCls) is Globals.Test:
            if Globals.__Test is None:
                # Create and remember instance
                Globals.__Test = Globals.Test()
                # Store instance reference as the only member in the handle
                self.__dict__['_Globals__Test'] = Globals.__Test
    

    """ def __getattr__(self, attr):
        # Delegate access to implementation 
        return getattr(self.__instance, attr)

    def __setattr__(self, attr, value):
        # Delegate access to implementation
        return setattr(self.__instance, attr, value) """



#!/usr/bin/env python3


"""Test result object"""



from Octopus.OctopusHTMLReport.OctopusTestCase import TestCase




class TestResult(object):
    # note: _TestResult is a pure representation of results.
    # It lacks the output and reporting ability compares to unittest._TextTestResult.

    def __init__(self):        
        self.success_count = 0
        self.failure_count = 0
        self.error_count = 0
        self.testsRun = 0
        self._TestCase = None
        

        # result is a list of result in 4 tuple
        # (
        #   result code (0: success; 1: fail; 2: error),
        #   TestCase object,
        # )
        self.result = []

    def startTest(self, testMethodName='testMethod'):
        self._TestCase = TestCase(testMethodName)
        self.testsRun += 1
        

    def stopTest(self):
        _status = 0        
        if self._TestCase.error_count > 0:
            self.error_count += 1
            _status = 2
        elif self._TestCase.failure_count > 0:
            self.failure_count += 1            
            _status = 1
        else:
            self.success_count += 1
            _status = 0
        self.result.append((_status,self._TestCase))
        self._TestCase = None
        
    def addSuccess(self, testDetails,screenShot):
        self._TestCase.addPassStep( testDetails,screenShot)        

    def addError(self, testDetails,screenShot):        
        self._TestCase.addErrorStep(testDetails,screenShot)
       
    def addFailure(self, testDetails,screenShot):        
        self._TestCase.addFailStep(testDetails,screenShot)
        

    def addInfo(self, testDetails):        
        self._TestCase.addInfoStep(testDetails)
       



#!/usr/bin/env python3

class Configuration:
  connectionStrings =	{    
    "ExcelConn":"Driver={Microsoft Excel Driver (*.xls, *.xlsx, *.xlsm, *.xlsb)};DBQ=[ControlFileName];",
  }

  appSettings =	{
    "ControlFileName": "\\resources\\TestData\\ControlFile.xlsx",
    "Browser.DefaultTimeout": "10",
    "ChromeDriverPath": "\\resources\\ChromeDriver\\chromedriver.exe",
    "FirFoxDriverPath": "\\resources\\FireFoxDriver\\geckodriver.exe",
    "IEedgeDriverPath": "\\resources\\IEedgeDriver\\MicrosoftWebDriver.exe",
    "ReportPath":"\\reports"
  }





#!/usr/bin/env python3

import os
import sys

import pyodbc
from Octopus.OctopusCore.configuration import Configuration


class DataManager:
        cfg = Configuration()
        qry=""
        @staticmethod
        def getDictionaryTableFromExcell(strQuery):
                dataTable=dict()
                colNmaes=dict()
                rowID = 0
                colID =0
                cnxn=''
                cursor=None
                try:
                        DataManager.qry=strQuery
                        cStr = DataManager.cfg.connectionStrings["ExcelConn"].replace("[ControlFileName]",os.getcwd()+DataManager.cfg.appSettings["ControlFileName"])
                        cnxn = pyodbc.connect(cStr,autocommit=True)
                        cursor = cnxn.cursor()
                        cursor.execute(DataManager.qry)
                        # create a dictionary of column names
                        for desc in cursor.description:
                                colNmaes[colID]=desc[0]
                                colID+=1   
                        # Loop through returned data rows                                             
                        for row in cursor: 
                                colID =0   
                                dataRow=dict() 
                                #  create a dictionary using column name and value
                                for column in row:     
                                        dataRow[colNmaes[colID]]= str(column)
                                        colID+=1  
                                # Add the dictionary to the table dictionary
                                dataTable[rowID] = dataRow
                                rowID+=1     
                except:
                        print("Failed to execute query ({0}) on database for Error: {1}".format(strQuery,sys.exc_info()[1]))
                        cnxn.rollback()
                finally:                       
                        cursor.close()
                        cnxn.close()
                        return dataTable
        
        @staticmethod
        def getDictionaryFrom2ColumnsInExcellSheet(strQuery):
                dataTable=dict()  
                cnxn=''  
                cursor=None                            
                try:
                        DataManager.qry=strQuery
                        cStr = DataManager.cfg.connectionStrings["ExcelConn"].replace("[ControlFileName]",os.getcwd()+DataManager.cfg.appSettings["ControlFileName"])
                        cnxn = pyodbc.connect(cStr,autocommit=True)
                        cursor = cnxn.cursor()
                        cursor.execute(DataManager.qry)                       
                        # Loop through returned data rows                                             
                        for row in cursor: 
                                dataTable[row[0]] = row[1]
                                  
                except:
                        print("Failed to execute query ({0}) on database for Error: {1}".format(strQuery,sys.exc_info()[1]))
                        cnxn.rollback()
                finally:                       
                        cursor.close()
                        cnxn.close()
                        return dataTable

        @staticmethod
        def executeQueryOnExcell(strQuery): 
                cnxn=''
                cursor=None              
                try:
                        DataManager.qry=strQuery
                        cStr = DataManager.cfg.connectionStrings["ExcelConn"].replace("[ControlFileName]",os.getcwd()+DataManager.cfg.appSettings["ControlFileName"])
                        cnxn = pyodbc.connect(cStr,autocommit=True)
                        cursor = cnxn.cursor()
                        cursor.execute(DataManager.qry)
                        print("query (%s) executed successfully.!"%(strQuery) )                   
                except:
                        print("Failed to execute query ({0}) on database for Error: {1}".format(strQuery,sys.exc_info()[1]))
                        cnxn.rollback()
                finally:
                        cursor.close()
                        cnxn.close()
                        print("connection is closed")                                
        @staticmethod
        def getDictionaryTableFromSource(strConn,strQuery):
                dataTable=dict()
                colNmaes=dict()
                rowID = 0
                colID =0
                cnxn=''
                cursor=None
                try:
                        DataManager.qry=strQuery
                        cStr = DataManager.cfg.connectionStrings[strConn]
                        cnxn = pyodbc.connect(cStr,autocommit=True)
                        cursor = cnxn.cursor()
                        cursor.execute(DataManager.qry)
                        # create a dictionary of column names
                        for desc in cursor.description:
                                colNmaes[colID]=desc[0]
                                colID+=1   
                        # Loop through returned data rows                                             
                        for row in cursor: 
                                colID =0   
                                dataRow=dict() 
                                #  create a dictionary using column name and value
                                for column in row:     
                                        dataRow[colNmaes[colID]]= str(column)
                                        colID+=1  
                                # Add the dictionary to the table dictionary
                                dataTable[rowID] = dataRow
                                rowID+=1          
                                              
                except:
                        print("Failed to execute query ({0}) on database for Error: {1}".format(strQuery,sys.exc_info()[1]))
                        cnxn.rollback()
                finally:
                        cursor.close()
                        cnxn.close()
                        return dataTable



#!/usr/bin/env python3
 
import datetime
import os
import sys

from selenium import webdriver

from Octopus.OctopusCore.configuration import Configuration
from Octopus.OctopusCore.dataManager import DataManager
from Octopus.OctopusCore.globals import Globals


class Reporter:
    #  This class will be responsible for reporting status to html report and collect test statistics.    
    
    @staticmethod
    def startTest(sTestCaseName):
        objFile=None
        try:
            print("\n\n_________ Start Test Case [ " + sTestCaseName + " ] _________")
            Globals.Report.StartTime = datetime.datetime.today().strftime("%Y-%m-%d %H:%M:%S")
            Globals.Report.TotalTestCaseCount += 1             
            objFile = open(Globals.Report.ReportFileName, 'a+')
            if objFile.writable:
                objFile.write("<HTML><head><meta charset=\"UTF-8\"></head><BODY><TABLE BORDER=1 CELLPADDING=3 CELLSPACING=1 WIDTH=100%>")
                            #   "<tr><td bgcolor="#5B2C6F" width="15%"><font face="Segoe UI Semilight" color="WHITE" size="3"><b>&nbsp;&nbsp;Test Case ID:</b></font></td><td bgcolor="#5B2C6F" colspan="5"><font face="Segoe UI Semilight" color="WHITE" size="5"><b><center>test case01</center></b></font></td></tr>"
                objFile.write("<TR><TD BGCOLOR=#5B2C6F WIDTH=15%><FONT FACE=\"Segoe UI Semilight\" COLOR=WHITE SIZE=3><B>&nbsp&nbspTest Case ID:</B></FONT></TD><TD BGCOLOR=#5B2C6F COLSPAN=5><FONT FACE=\"Segoe UI Semilight\" COLOR=WHITE SIZE=5><B><Center>" + sTestCaseName + "</Center></B></FONT></TD></TR>")
                Globals.Report.PassCount = 0
                Globals.Report.FailCount = 0  
            else:
                raise IOError("issue in writing to html report.")
        except:
            print("\t\t[Failed]  Starting New Test : {0}".format(sys.exc_info()[1]))         
        finally:
            objFile.close()

    @staticmethod
    def endTest():
        objFile=None
        try:
            print("\n_________ End Test Case _________")
            Globals.Report.EndTime = datetime.datetime.today().strftime("%Y-%m-%d %H:%M:%S")            
            objFile = open(Globals.Report.ReportFileName, 'a+')
            if objFile.writable:
                objFile.write("<TR COLS=5><TD BGCOLOR=#C39BD3 WIDTH=15%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B>&nbsp;&nbsp;Start Time: </B></FONT></TD><TD BGCOLOR=C39BD3 WIDTH=30%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B><Center>" + str(Globals.Report.StartTime) + "</Center></B></FONT></TD><TD BGCOLOR=C39BD3 WIDTH=15%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B>&nbsp;&nbsp;End Time: </B></FONT></TD><TD BGCOLOR=C39BD3 WIDTH=30%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B><Center>" + str(Globals.Report.EndTime) + "</Center></B></FONT></TD></TR>")
                if Globals.Report.FailCount > 0:
                    Globals.Report.FinalResults = "FAIL"
                    Globals.Report.TotalFailCount +=1
                    objFile.write("<TR COLS=5><TD BGCOLOR=#C39BD3 WIDTH=15%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B><Center>End of Test Case</Center></B></FONT></TD><TD BGCOLOR=C39BD3 WIDTH=30%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B><Center>Test Steps Passed : &nbsp;" + str(Globals.Report.PassCount) + "</Center></B></FONT></TD><TD BGCOLOR=C39BD3 WIDTH=15%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B><Center>Test Steps Failed : &nbsp;" + str(Globals.Report.FailCount) + "</Center></B></FONT></TD><TD BGCOLOR=C39BD3 WIDTH=30%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B><Center>Final Test Result : </B></FONT><FONT COLOR=RED><B>&nbsp;" + str(Globals.Report.FinalResults) + "</Center></B></FONT></TD></TR>")
                else:
                    Globals.Report.FinalResults = "PASS"
                    Globals.Report.TotalPassCount +=1
                    objFile.write("<TR COLS=5><TD BGCOLOR=#C39BD3 WIDTH=15%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B><Center>End of Test Case</Center></B></FONT></TD><TD BGCOLOR=C39BD3 WIDTH=30%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B><Center>Test Steps Passed : &nbsp;" + str(Globals.Report.PassCount) + "</Center></B></FONT></TD><TD BGCOLOR=C39BD3 WIDTH=15%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B><Center>Test Steps Failed :&nbsp;" + str(Globals.Report.FailCount) + "</Center></B></FONT></TD><TD BGCOLOR=C39BD3 WIDTH=30%><FONT FACE=\"Microsoft YaHei UI\" COLOR=WHITE SIZE=2><B><Center>Final Test Result : </B></FONT><FONT COLOR=#008000><B>&nbsp;" + str(Globals.Report.FinalResults) + "</Center></B></FONT></TD></TR>")

                objFile.write("</TABLE></BODY></HTML>")
                Globals.Report.PassCount = 0
                Globals.Report.FailCount = 0  
            else:
                raise IOError("issue in writing to html report.")
        except:
            print("\t\t[Failed]  Ending Test : {0}".format(sys.exc_info()[1]))         
        finally:
            objFile.close()

    @staticmethod
    def startIteration(sTestCaseName,iIterID):
        objFile=None
        try:
            print("\n\t............ Start Iteration With ROW [ " + str(iIterID) + " ]............\n")                            
            objFile = open(Globals.Report.ReportFileName, 'a+')
            if objFile.writable:
                objFile.write("<TR COLS=6><TD BGCOLOR=#3498DB WIDTH=15%><FONT FACE=\"Segoe UI Semilight\" COLOR=WHITE SIZE=3><B>&nbsp;&nbsp;Test Case Description:</B></FONT></TD><TD BGCOLOR=#3498DB WIDTH=30%><FONT FACE=\"Segoe UI Semilight\" COLOR=WHITE SIZE=3><B><Center>" + sTestCaseName + "</Center></B></FONT></TD><TD BGCOLOR=#3498DB WIDTH=15%><FONT FACE=\"Segoe UI Semilight\" COLOR=WHITE SIZE=3><B>&nbsp;&nbsp;Iteration Id: </B></FONT></TD><TD BGCOLOR=#3498DB WIDTH=30%><FONT FACE=\"Segoe UI Semilight\" COLOR=WHITE SIZE=3><B><Center>" + str(iIterID) + "</Center></B></FONT></TD></TR>")
                objFile.write("<TR COLS=6><TD BGCOLOR=#3498DB WIDTH=15%><FONT FACE=\"Segoe UI Semilight\" COLOR=WHITE SIZE=3><B>&nbsp;&nbsp;Step Status</B></FONT></TD><TD BGCOLOR=#3498DB COLSPAN=5><FONT FACE=\"Segoe UI Semilight\" COLOR=WHITE SIZE=3><B>&nbsp;&nbsp;Step Details</B></FONT></TD></TR>")
            else:
                raise IOError("issue in writing to html report.")
        except:
            print("\t\t[Failed]  Starting New Iteration : {0}".format(sys.exc_info()[1]))         
        finally:
            objFile.close()

    @staticmethod
    def endIteration():
        try:
            print("\n\t............ End Iteration ............")                                       
        except:
            print("\t\t[FAIL]  Ending  Iteration  : {0}".format(sys.exc_info()[1]))    

    @staticmethod
    def writeFinalStatus():
        objFile=None
        try:            
            objFile = open(Globals.Report.ReportFileName, 'a+')
            if objFile.writable:
                objFile.write("<TABLE><TR><TD COLSPAN=2> </TD></TR></TABLE>")
                objFile.write("<HTML><head><meta charset=\"UTF-8\"></head><BODY><TABLE BORDER=1 CELLPADDING=3 CELLSPACING=1 WIDTH=100%><br/><br/><br/><Caption><font FACE=\"Microsoft YaHei UI\" COLOR=#009900 SIZE=3><b> Test Automation Result Summary </b></font></caption>")
                objFile.write("<TR COLS=6><TD BGCOLOR=#6d7b8d WIDTH=33%><FONT FACE=\"Microsoft YaHei UI\" COLOR=White SIZE=2><B><Center>Total No Of Test Cases</Center></B></FONT></TD><TD BGCOLOR=#6d7b8d WIDTH=33%><FONT FACE=\"Microsoft YaHei UI\" COLOR=White SIZE=2><B><Center>Passed</Center></B></FONT></TD><TD BGCOLOR=#6d7b8d WIDTH=33%><FONT FACE=\"Microsoft YaHei UI\" COLOR=White SIZE=2><B><Center>Failed</Center></B></FONT></TD></TR>")
                objFile.write("<TR><TD BGCOLOR=#6d7b8d WIDTH=33%><FONT FACE=\"Microsoft YaHei UI\" COLOR=White SIZE=2><B><Center>" + str(Globals.Report.TotalTestCaseCount) + "</Center></B></FONT></TD><TD BGCOLOR=#6d7b8d WIDTH=33%><FONT FACE=\"Microsoft YaHei UI\" COLOR=#00E600 SIZE=2><B> <Center>" + str(Globals.Report.TotalPassCount) + "</Center></B></FONT></TD><TD BGCOLOR=#6d7b8d WIDTH=33%><FONT FACE=\"Microsoft YaHei UI\" COLOR=Red SIZE=2><B> <Center>" + str(Globals.Report.TotalFailCount) + "</Center></B></FONT></TD></TR>")
                objFile.write(" </table></body></html>")
            else:
                raise IOError("issue in writing to html report.")
            
            if Globals.Report.TotalFailCount > 0:
                Globals.Report.FinalResults = "FAIL"
                print("\n\n___Executed:"+str(Globals.Report.TotalTestCaseCount)+", Passed:"+str(Globals.Report.TotalPassCount)+", Failed:"+str(Globals.Report.TotalFailCount)+
                    " ______ Test Status [ FAIL ] _________\n")
            else:
                Globals.Report.FinalResults = "PASS" 
                print("\n\n___ Executed: "+str(Globals.Report.TotalTestCaseCount)+", passed: "+str(Globals.Report.TotalPassCount)+", failed: "+str(Globals.Report.TotalFailCount)+
                    " ______ Test Status [ PASS ] _________\n")          

        except:
            print("\t\t[Failed]  Reporting Test Status: {0}".format(sys.exc_info()[1]))         
        finally:
            objFile.close()     
    
    @staticmethod
    def passed(sDetails):
        objFile=None
        try:
            print("\t\t[PASS]  " + sDetails)
            Globals.Report.PassCount +=1
            objFile = open(Globals.Report.ReportFileName, 'a+')
            if objFile.writable:
                screenFile = Reporter.saveScreenShot()                
                objFile.write("<TR COLS=5><TD BGCOLOR=#EEEEEE WIDTH=15%><FONT FACE=\"Times New Roman (Headings CS)\" SIZE=3 COLOR=#008000><Center><b>PASS</b></Center></FONT></TD><TD BGCOLOR=#EEEEEE COLSPAN=5><FONT FACE=\"WINGDINGS\"  SIZE=4>2</FONT><FONT FACE=\"Times New Roman (Headings CS)\" SIZE=2><A HREF='" + screenFile + "'>&nbsp;" + sDetails + "</A></FONT></TD></TR>")
            else:
                raise IOError("issue in writing to html report.")
        except:
            print("\t\t[Failed]  reporting status : {0}".format(sys.exc_info()[1]))         
        finally:
            objFile.close()
    
    @staticmethod
    def failed(sDetails):
        objFile=None
        try:
            print("\t\t[FAIL]  " + sDetails)
            Globals.Report.FailCount +=1
            objFile = open(Globals.Report.ReportFileName, 'a+')
            if objFile.writable:
                screenFile = Reporter.saveScreenShot()                
                objFile.write("<TR COLS=5><TD BGCOLOR=#EEEEEE WIDTH=15%><FONT FACE=\"Times New Roman (Headings CS)\" SIZE=3 COLOR=RED><Center><b>FAIL</b></Center></FONT></TD><TD BGCOLOR=#EEEEEE COLSPAN=5><FONT FACE=\"WINGDINGS\"  SIZE=4>2</FONT><FONT FACE=\"Times New Roman (Headings CS)\" SIZE=2><A HREF='" + screenFile + "'>&nbsp;" + sDetails + "</A></FONT></TD></TR>")
                            
            else:
                raise IOError("issue in writing to html report.")
        except:
            print("\t\t[Failed]  reporting status : {0}".format(sys.exc_info()[1]))         
        finally:
            objFile.close()

    @staticmethod
    def info(sDetails):
        objFile=None
        try:
            print("\t\t[INFO]  " + sDetails)            
            objFile = open(Globals.Report.ReportFileName, 'a+')
            if objFile.writable:                
                objFile.write("<TR COLS=5><TD BGCOLOR=#EEEEEE WIDTH=15%><FONT FACE=\"Times New Roman (Headings CS) \" SIZE=3 COLOR=#FF8C00></FONT><FONT FACE=\"Times New Roman (Headings CS)\"  SIZE=3 COLOR=#0000FF><Center><b>INFO</b></Center></FONT></TD><TD BGCOLOR=#EEEEEE COLSPAN=5><FONT FACE=\"Times New Roman (Headings CS)\"  SIZE=2>" + sDetails + "</FONT></TD></TR>")
            else:
                raise IOError("issue in writing to html report.")
        except:
            print("\t\t[Failed]  reporting status : {0}".format(sys.exc_info()[1]))         
        finally:
            objFile.close()

    @staticmethod
    def startHtmlReport(sProjectName):
        objFile=None
        try:
            Globals.Report.PassCount = 0
            Globals.Report.FailCount = 0
            Globals.Report.ScreenCaptureCount = 0
            Globals.Report.TotalPassCount = 0
            Globals.Report.TotalFailCount = 0
            Globals.Report.TotalTestCaseCount = 0      
            
            objFile = open(Globals.Report.ReportFileName, 'w+')
            if objFile.writable:
                objFile.write("<HTML><head><meta charset=\"UTF-8\"></head><BODY><TABLE BORDER=0 CELLPADDING=3 CELLSPACING=1 WIDTH=100%>")
                objFile.write("<TR COLS=2><TD BGCOLOR=WHITE ><IMG SRC=http://www.elm.sa/_LAYOUTS/AlElmPortal/Images/elm_logo.png> </TD> <TD WIDTH=100% BGCOLOR=WHITE><Center><FONT FACE=\"Microsoft YaHei UI\" COLOR=#088A08 SIZE=4><B>&nbsp;" + sProjectName + "  Automation Execution Report <br/>" + datetime.datetime.today().strftime("%Y-%m-%d") + " - " + datetime.datetime.today().strftime("%H:%M:%S") + " <br/> Tested on Machine " + os.getenv("COMPUTERNAME") + "</B></FONT></Center></TD></TR></TABLE>")
                objFile.write("</BODY></HTML><BR>")
            else:
                raise IOError("issue in writing to html report.")
        except:
            print("\t\t[Failed]  opening HTML Report File : {0}".format(sys.exc_info()[1]))         
        finally:
            objFile.close()

    @staticmethod
    def saveScreenShot():
        imgPath=""
        try:
            if Globals.Test.Browser != None:
                Globals.Report.ScreenCaptureCount+=1
                imgPath=Globals.Report.ScreenShootFolder+"\\Img_"+str(Globals.Report.ScreenCaptureCount)+".png"
                Globals.Test.Browser.save_screenshot(imgPath)
                imgPath = Globals.Report.ScreenShootHTMLPath+"\\Img_"+str(Globals.Report.ScreenCaptureCount)+".png"
        except:
            print("\t\t[Failed]  Take a screen shoot : {0}".format(sys.exc_info()[1]))
        finally:
            return imgPath



#!/usr/bin/env python3


"""Test case implementation"""


import datetime




class TestCase(object):
    """A class whose instances are single test cases.
    that hold all information about a test case like name, description
    and steps count, details 
    and statistics like totla pass, fail and error steps
    """

    def __init__(self, testMmethodName='runTest',testCaseDescription='desc'):        
        self.methodName = testMmethodName
        self.description = testCaseDescription        
        self.success_count = 0
        self.failure_count = 0
        self.error_count = 0        
        # steps is a list of result in 4 tuple
        # (
        #   step type code (0: success; 1: fail; 2: error, 3: info),
        #   timestamp time,
        #   step details string,
        #   sceenchoot string path,
        # )
        self.steps = []
        self.steps_count = 0

    def addInfoStep(self, details):
        self.success_count += 1
        self.steps_count += 1
        tiemStamp = str(datetime.datetime.now().time()).split('.')[0]  
        self.steps.append((3,tiemStamp,details,''))

    def addPassStep(self, details,screenshot=''):
        self.success_count += 1
        self.steps_count += 1
        tiemStamp = str(datetime.datetime.now().time()).split('.')[0]
        self.steps.append((0,tiemStamp,details,screenshot))

    def addFailStep(self, details, screenshot=''):
        self.failure_count += 1
        self.steps_count += 1
        tiemStamp = str(datetime.datetime.now().time()).split('.')[0]
        self.steps.append((1,tiemStamp,details,screenshot))

    def addErrorStep(self, details,screenshot=''):
        self.error_count += 1
        self.steps_count += 1        
        tiemStamp = str(datetime.datetime.now().time()).split('.')[0]
        self.steps.append((2,tiemStamp,details,screenshot))

    def shortDescription(self):
        doc = self.description
        return doc and doc.split("\n")[0].strip() or None


    def id(self):
        return  self.methodName

    def __eq__(self, other):
        if type(self) is not type(other):
            return NotImplemented

        return self.methodName == other.methodName

    def __hash__(self):
        return hash((type(self), self.methodName))

    def __str__(self):
        return "(%s)" % (self.methodName)

    def __repr__(self):
        return "<testMethod=%s>" % \
               (self.methodName)

    
    

   


#!/usr/bin/env python3

import datetime
import os
import sys

from Octopus.OctopusCore.browsing import Browsing
from Octopus.OctopusCore.configuration import Configuration
from Octopus.OctopusCore.dataManager import DataManager
from Octopus.OctopusCore.globals import Globals
from Octopus.OctopusCore.executer import Executer
from Octopus.OctopusHTMLReport.OctopusReporter import Reporter


class Driver:
    # this clas is the core of the frmaework and will be responsible for mabaging test environment , test data, and test scenarios.
    tblDriver=dict()
    tblEnvironment=dict()
    query = ""
    TestEnvironment=""
    RQMID = ""

    @staticmethod
    def runSequentialTest():
        try:

            # Initializing all global variables and report configurations
            tblEnvironment = DataManager.getDictionaryFrom2ColumnsInExcellSheet("select Name,Value from [Environment$] WHERE Name <> \'\'")            
            Globals.Test.initializeTestGlobals(tblEnvironment)
            Globals.Report.initializeRepoeterGlobals(tblEnvironment)

            # Get the RQM Variables             
            if os.getenv("qm_RQM_TESTCASE_WEBID") == None:
                RQMID="" 
            else :
                RQMID = os.getenv("qm_RQM_TESTCASE_WEBID")

            if RQMID == "":
                query = "select * from [Driver$] where Execution_Flag in ('Yes','yes','YES')"
            else :
                query = "select * from [Driver$] where RQMID = '" + RQMID + "' And Execution_Flag in ('Yes','yes','YES')"
            
            # get all selected test cases for execution in Driver sheet
            tblDriver = DataManager.getDictionaryTableFromExcell(query)

            if len(tblDriver) == 0:
                raise Exception("There is no Data in Driver Sheet for query ("+query+")")
            
            # Opening the report HTML file
            Reporter.openReport(Globals.Report.ReportFileName,Globals.Test.ProjectName)

            # Opening the test environment [Browser]
            Globals.Test.Browser = Browsing.openBrowser(Globals.Test.BrowserName,Globals.Test.URL)

            # Loop through the Driver sheet test functions with Execution_Flag = YES 
            for row in tblDriver:
                Executer.executeTestCase(tblDriver[row])

            Reporter.writeReport()

            Browsing.closeBrowser()

        except:
            if Globals.Test.Browser != None:
                Browsing.closeBrowser()
            print("\t\t Driver Error <<<{0}>>> ".format(sys.exc_info()[1]))
            print("\n\n ========= END OF EXEC ========= Test Status [FAIL] =========")


#!/usr/bin/env python3
import sys

from Octopus.OctopusCore.common import Common
from Octopus.OctopusCore.globals import Globals
from Octopus.OctopusHTMLReport.OctopusReporter import Reporter
from Octopus.OctopusCore.dataManager import DataManager
from test.scenarios.testScenarios import TestScenarios

class Executer:
    # This calss will be responsible for invoking the test methods recived from driver.
    RowID=""
    TestCaseID=""    
    Function_Name=""
    SheetName=""
    SheetRowNo=""    
    Status=""
    StartTime=""
    EndTime=""
    TestDataRow=None


    @staticmethod
    def executeTestCase(row):
        lstRowsNo = list()
        try:
            # Initialize variables by current Driver Sheet row
            Function_Name = str(row["Function_Name"])
            SheetName = str(row["TestDataSheetName"])
            RowNo = str(row["TestDataSheetRowNo"])            
            RowID = str(row["RowID"])                        
            TestCaseID = str(row["TestCaseID"])           
            TestDataRow = None
            testFunction = None

            lstRowsNo = Common.GetIterations(RowNo,',')

            if len(lstRowsNo) < 1:
                raise Exception("Test Data Sheet RowNo is empty for function ({0}) in Driver sheet row ({1})".format(Function_Name,RowID))
            else:
                Globals.Report.TestCaseIterationsCount = len(lstRowsNo)
                Globals.Report.TestCaseStatus = True
                # Reporter.startTest(TestCaseID)
                for rowNum in lstRowsNo:                    
                    # Reporter.startIteration(Function_Name, rowNum)
                    Reporter.startTest(Function_Name)
                    TestDataRow = DataManager.getDictionaryTableFromExcell("select * from [" + SheetName + "$] where RowID=" + rowNum )[0]                    
                    if TestDataRow != None:
                        if hasattr(TestScenarios,Function_Name):
                            testFunction = getattr(TestScenarios,Function_Name)
                            testFunction(TestDataRow)
                        else:
                            raise Exception("TestFunction ( {0} ) Not Exist In TestScenarios".format(Function_Name))  
                    else:
                        raise Exception("Test Data Row ( {0} ) Not Exist In Sheet ( {1} )".format(rowNum,SheetName))                        

                    Reporter.stopTest()
                    # Reporter.endIteration()
                
                # Reporter.stopTest()

        except:
            print("Failed to execute Test Case for Error {0}".format(sys.exc_info()[1]))
        finally:
            pass


#!/usr/bin/env python3

import os

from selenium import webdriver
from selenium.common.exceptions import (ElementNotVisibleException,
                                        NoSuchElementException,
                                        StaleElementReferenceException,
                                        TimeoutException, WebDriverException)
from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support import expected_conditions as EC
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.support.ui import Select

from Octopus.OctopusCore.globals import Globals
from Octopus.OctopusHTMLReport.OctopusReporter import Reporter


def findElementsBy(self: (WebElement  or webdriver),by=By.ID, value=None):    
    try:
        # return self.find_elements(by,value)
        return WebDriverWait(Globals.Test.Browser, 10).until(EC.presence_of_all_elements_located((by,value)))
    except NoSuchElementException:
        return WebElement(self,id(self))
    except TimeoutException:
        return WebElement(self,id(self))
    except StaleElementReferenceException:
        return WebElement(self,id(self))
    except WebDriverException:
        return WebElement(self,id(self))

def findElementBy(self: (WebElement or webdriver),by=By.ID, value=None):    
    try:
        # return self.find_element(by,value)
        return WebDriverWait(Globals.Test.Browser, 10).until(EC.presence_of_element_located((by,value)))
    except NoSuchElementException:
        return WebElement(self,id(self))    
    except StaleElementReferenceException:
        return WebElement(self,id(self))
    except WebDriverException:
        return WebElement(self,id(self))
    
    
def isDisplayed(self: WebElement):
    try:
        if self is not None:
            return self.is_displayed()
        else:
            return False
    except NoSuchElementException:
        return False
    except WebDriverException:
        return False
    except Exception:
        return False

def Clear(self: WebElement):
    try:
        if self is not None:
            self.clear()
        else:
            return False
    except NoSuchElementException:
        return False
    except WebDriverException:
        return False
    except Exception:
        return False

def sendKeys(self: WebElement, *value):
    try:
        if self is not None:
            self.clear()
            self.send_keys(value)
            strElement = get_element_string(self)
            Reporter.info("Sent Data To Element  ***<<((   %s   ))>>***" % strElement)
        else:
            return False
    except NoSuchElementException:
        return False
    except WebDriverException:
        return False
    except Exception:
        return False

def Click(self: WebElement):
    try:
        if self is not None:
            self.click()
            strElement = get_element_string(self)
            Reporter.info("Clicked   On Element  ***<<((   %s   ))>>***" % strElement)
        else:
            return False
    except NoSuchElementException:
        return False
    except WebDriverException:
        return False
    except Exception:
        return False

def isSelected(self: WebElement):
    try:
        if self is not None:
            return self.is_selected()
        else:
            return False
    except NoSuchElementException:
        return False
    except WebDriverException:
        return False
    except Exception:
        return False

def isEnabled(self: WebElement):
    try:
        if self is not None:
            return self.is_enabled()
        else:
            return False
    except NoSuchElementException:
        return False
    except WebDriverException:
        return False
    except Exception:
        return False




def waitUntilHidden(self:WebElement,timeout:int=5):
    try:        
        elm = None
        elm = WebDriverWait(Globals.Test.Browser, timeout, 1, (StaleElementReferenceException,NoSuchElementException)).until(EC.invisibility_of_element(self))
        if elm is not None :
            return True
        else:
            return False
    except TimeoutException:
        return False
    except Exception:
        return False
    
def waitUntilDisplay(self:WebElement,timeout:int=5):
    try:        
        elm = None
        elm = WebDriverWait(Globals.Test.Browser, timeout, 1, (StaleElementReferenceException,NoSuchElementException)).until(EC.visibility_of(self))
        
        if elm is not None :
            return True
        else:
            return False
    except TimeoutException:
        return False
    except Exception:
        return False

def waitUntilExistInDOM(self:WebElement,timeout:int=5):
    try:
        elm = None
        elm = WebDriverWait(Globals.Test.Browser, timeout, 1, (StaleElementReferenceException,NoSuchElementException)).until(EC.presence_of_element_located(self))
        if elm is not None:
            return True
        else:
            return False
    except TimeoutException:
        return False
    except Exception:
        return False


def multi_select(self:WebElement, options):
      try:

        if self is not None:            
            select = Select(self)
            for option in options:
                select.select_by_visible_text(option)
        else:
            return False
        
      except NoSuchElementException:
          return False
      except WebDriverException:
          return False
      except Exception:
          return False

def select(self:WebElement, option):
      try:

        if self is not None:            
            select = Select(self)
            select.select_by_visible_text(option)
        else:
            return False
        
      except NoSuchElementException:
          return False
      except WebDriverException:
          return False
      except Exception:
          return False

def get_element_string(element):
    elementid = element.get_attribute("id")
    elementName = element.get_attribute("name")
    elementClass = element.get_attribute("class")
    elementType = element.get_attribute("type")
    elementTag = element.tag_name
    if(elementid is not None and len(elementid) > 0):
        return "ID: "+elementid
    elif (elementName is not None and len(elementName) > 0):
        return "Name: "+elementName
    elif (elementClass is not None and len(elementClass) > 0):
        return "Class: "+elementClass
    elif (elementType is not None and len(elementType) > 0):
        return "Type: "+elementType
    elif (elementTag is not None and len(elementTag) > 0):
        return "Tag: "+elementTag
    else:
        return "-"

WebDriver.findElementBy = findElementBy
WebDriver.findElementsBy = findElementsBy


WebElement.multi_select = multi_select
WebElement.select = select
WebElement.findElementBy = findElementBy
WebElement.findElementsBy = findElementsBy
WebElement.isEnabled = isEnabled
WebElement.isSelected = isSelected
WebElement.Click = Click
WebElement.sendKeys = sendKeys
WebElement.Clear = Clear
WebElement.isDisplayed = isDisplayed
WebElement.waitUntilHidden = waitUntilHidden
WebElement.waitUntilExistInDOM = waitUntilExistInDOM
WebElement.waitUntilDisplay = waitUntilDisplay




#!/usr/bin/env python3
import os
import sys

from selenium import webdriver
from selenium.webdriver.support.events import EventFiringWebDriver

from Octopus.OctopusCore.configuration import Configuration
from Octopus.OctopusCore.WebEventListener import WebEventListener



class Browsing:

    @staticmethod
    def openBrowser(sBrowserType,sUrl):
       try:
           
           if str(sBrowserType).lower()=="chrome":
               driver = webdriver.Chrome(executable_path=os.getcwd()+Configuration.appSettings["ChromeDriverPath"])
               Browsing.Driver = EventFiringWebDriver(driver, WebEventListener())                
               Browsing.Driver.maximize_window()
               Browsing.Driver.implicitly_wait(5)
               Browsing.Driver.timeout = 5 
               Browsing.Driver.get(sUrl)
               
           elif str(sBrowserType).lower()=="firefox":
               driver = webdriver.Firefox(executable_path=os.getcwd()+Configuration.appSettings["FirFoxDriverPath"])
               Browsing.Driver = EventFiringWebDriver(driver, WebEventListener())  
               Browsing.Driver.maximize_window()
               Browsing.Driver.implicitly_wait(5)
               Browsing.Driver.timeout = 5 
               Browsing.Driver.get(sUrl)
           else:
               driver = webdriver.Edge(executable_path=os.getcwd()+Configuration.appSettings["IEedgeDriverPath"])
               Browsing.Driver = EventFiringWebDriver(driver, WebEventListener())  
               Browsing.Driver.maximize_window()
               Browsing.Driver.implicitly_wait(5)
               Browsing.Driver.timeout = 5 
               Browsing.Driver.get(sUrl)
                
       except:
           print("Failed to open browser ({0})  for Error: {1}".format(sBrowserType,sys.exc_info()[1]))     
       finally:                      
           return Browsing.Driver
    
    @staticmethod
    def closeBrowser():
        try:
            Browsing.Driver.quit()
        except:
           print("Failed to close browser, for Error: {0}".format(sys.exc_info()[1]))           
        



""" p = Browsing()
p.openBrowser("chrome","http://google.com")
p.closeBrowser() """

""" print(dir(p))
x = getattr(Browsing,"openBrowser")
x(Browsing,"ie","http://google.com")

y = getattr(Browsing,"closeBrowser")
y(Browsing) """



#!/usr/bin/env python3
import time
from selenium.webdriver.support.event_firing_webdriver import EventFiringWebElement
from selenium.webdriver.support.events import EventFiringWebDriver, AbstractEventListener
from Octopus.OctopusHTMLReport.OctopusReporter import Reporter


class WebEventListener(AbstractEventListener):

    # def clear(self):
    #     # self._dispatch("change_value_of", (self._webelement, self._driver), "clear", ())
    #     pass

    def before_click(self, element, driver):
        pass
        # strElement = get_element_string(element)
        # Reporter.info("Clicked   On Element  ***<<((   %s   ))>>***" % strElement)

    # def after_click(self, element, driver):
    #     strElement = get_element_string(element)
    #     Reporter.info("Clicked   On Element  ***<<((  %s  ))>>***" % strElement)

    def after_change_value_of(self, element, driver):
        pass
        # strElement = get_element_string(element)
        # Reporter.info("Sent Data To Element  ***<<((   %s   ))>>***" % strElement)






def get_element_string(element):
        elementid = element.get_attribute("id")
        elementName = element.get_attribute("name")
        elementClass = element.get_attribute("class")
        elementType = element.get_attribute("type")
        elementTag = element.tag_name
        if(elementid is not None and len(elementid) > 0):
            return "ID: "+elementid
        elif (elementName is not None and len(elementName) > 0):
            return "Name: "+elementName
        elif (elementClass is not None and len(elementClass) > 0):
            return "Class: "+elementClass
        elif (elementType is not None and len(elementType) > 0):
            return "Type: "+elementType
        elif (elementTag is not None and len(elementTag) > 0):
            return "Tag: "+elementTag
        else:
            return "-"



from sys import setprofile
from Octopus.OctopusHTMLReport.OctopusTestResult import TestResult
from Octopus.OctopusHTMLReport.OctopusHTMLTestReport import HTMLTestReport
from Octopus.OctopusCore.globals import Globals
import datetime
import os
import sys
from enum import Enum


class Status(Enum):
    PASS = 'PASS'
    FAIL = 'FAIL'
    FATAL = 'FATAL'
    ERROR = 'ERROR'
    WARNING = 'WARNING'
    INFO = 'INFO'
    DEBUG = 'DEBUG'
    SKIP = 'SKIP'


class Reporter (object):
    _testResult = None
    _htmlReport = None
    _reportFile = None

    def __init__(self):
        """ Create singleton instance """
        # Check whether we already have an instance        
        if Reporter._testResult is None:
            # Create and remember instance
            Reporter._testResult = TestResult()
            # Store instance reference as the only member in the handle
            self.__dict__['_Reporter__Result'] = Reporter._testResult
    
        if Reporter._htmlReport is None:
            # Create and remember instance
            Reporter._htmlReport = TestResult()
            # Store instance reference as the only member in the handle
            self.__dict__['_Reporter__htmlReport'] = Reporter._htmlReport


        if Reporter._reportFile is None:
            # Create and remember instance
            Reporter._reportFile = TestResult()
            # Store instance reference as the only member in the handle
            self.__dict__['_Reporter__reportFile'] = Reporter._reportFile
    

    @staticmethod
    def openReport( sReportPath, sProjectName):
        try:
            Reporter._testResult = TestResult()
            Reporter._reportFile = open(sReportPath, 'wb')
            Reporter._htmlReport = HTMLTestReport(stream=Reporter._reportFile,
                                              title=sProjectName,
                                              description='Octopus HTML Test Report : {0}'.format(sProjectName))
        except:
            print("\t\t[ERROR]  opening HTML Report File : {0}".format(
                sys.exc_info()[1]))

    @staticmethod
    def writeReport():
        try:
            Reporter._htmlReport.generateReport(Reporter._testResult)
        except:
            print("\t\t[ERROR]  Writing HTML Report File : {0}".format(
                sys.exc_info()[1]))
        finally:
            print('\n\n=============================================================')
            print('Total tests run: {0}, Failures: {1}, Errors: {2}, Passed: {3}'.format(Reporter._testResult.testsRun, \
                    Reporter._testResult.failure_count,Reporter._testResult.error_count,Reporter._testResult.success_count))
            print('\nTime Elapsed: {0}'.format( (Reporter._htmlReport.stopTime-Reporter._htmlReport.startTime) + datetime.timedelta(0, 1)))
            print('=============================================================\n') 
            Reporter._reportFile.close()

    @staticmethod
    def getBrowserImage(Browser,imgPath):
        _imgPath = ""
        try:
            if Browser != None:
                Browser.save_screenshot(imgPath)
                _imgPath = imgPath
        except:
            print("\t\t[Failed]  Take a screen shoot : {0}".format(
                sys.exc_info()[1]))
        finally:
            return _imgPath
    
    @staticmethod
    def saveScreenShot():
        imgPath=""
        try:
            if Globals.Test.Browser != None:
                Globals.Report.ScreenCaptureCount+=1
                imgPath=Globals.Report.ScreenShootFolder+"\\Img_"+str(Globals.Report.ScreenCaptureCount)+".png"
                Globals.Test.Browser.save_screenshot(imgPath)
                imgPath = Globals.Report.ScreenShootHTMLPath+"\\Img_"+str(Globals.Report.ScreenCaptureCount)+".png"
        except:
            print("\t\t[Failed]  Take a screen shoot : {0}".format(sys.exc_info()[1]))
        finally:
            return imgPath

    @staticmethod
    def log( status, details):
        try:

            if(None == details):
                details = ""

            if (("<html" in details or "<HTML" in details or "</" in details) and (details.strip().endswith(">"))):
                details = "<textarea readonly style=\"height:150px\">" + details + "</textarea>"

            if(status is Status.FAIL):
                print("\t\t[ {0} ]  {1} .".format(Status.FAIL.value, details))
                imgPath = Reporter.saveScreenShot()
                Reporter._testResult.addFailure(details, imgPath)
            elif(status is Status.ERROR):
                imgPath = Reporter.saveScreenShot()
                print("\t\t[ {0} ]  {1} .".format(Status.ERROR.value, details))
                Reporter._testResult.addError(details, imgPath)
            elif(status is Status.PASS):
                imgPath = Reporter.saveScreenShot()
                print("\t\t[ {0} ]  {1} .".format(Status.PASS.value, details))
                Reporter._testResult.addSuccess(details, imgPath)
            else:
                print("\t\t[ {0} ]  {1} .".format(Status.INFO.value, details))
                Reporter._testResult.addInfo(details)

        except Exception:
            print("Reporter Log Function Failed. {0}".format(
                sys.exc_info()[1]))
            Reporter._testResult.addError(sys.exc_info()[1], '')

    @staticmethod
    def startTest( sTestCaseName):
        try:
            print(
                "\n\n_________ Start Test Case [ " + sTestCaseName + " ] _________")
            Reporter._testResult.startTest(sTestCaseName)
        except:
            print("\t\t[Failed]  Starting New Test : {0}".format(
                sys.exc_info()[1]))

    @staticmethod
    def stopTest():
        try:
            print("\n_________ End Test Case _________")
            Reporter._testResult.stopTest()
        except:
            print("\t\t[Failed]  Ending Test : {0}".format(sys.exc_info()[1]))

    @staticmethod
    def info( details):
        Reporter.log(Status.INFO, details)

    @staticmethod
    def passed( details):
        Reporter.log(Status.PASS, details)

    @staticmethod
    def failed( details):
        Reporter.log(Status.FAIL, details)

    @staticmethod
    def error( details):
        Reporter.log(Status.ERROR, details)




#!/usr/bin/env python3

import sys

class Common:

    @staticmethod
    def SwapArgs(_min, _max):
        swaplist=list()
        try:            
            __min = _max
            __max = _min
            swaplist = {str(__min),str(__max)}
        except:
            print("Failed to swap arguments for Error ({0})".format(sys.exc_info()[1]))
        finally:
            return swaplist
        

    @staticmethod
    def GetIterations( sIterations, Separator):
        lstIterations = list()
        lstSwap=list()
        arrTmp = str(sIterations).split(Separator)
        for num in arrTmp:
            arrRange = str(num).split('-')
            if len(arrRange) > 1:
                _min = arrRange[0]
                _max = arrRange[1]
                if int(_min) > int(_max):
                    lstSwap = Common.SwapArgs(_min,_max)
                    _min = lstSwap[0]
                    _max = lstSwap[1]

                for j in range(int(_min),(int(_max)+1)):
                    lstIterations.append(str(j))
            else :
                lstIterations.append(str(num))
        try:
            pass
        except:
            print("Failed to get Iterations from ({0}) for Error ({1})".format(sIterations,sys.exc_info()[1]))
        finally:
            return lstIterations

    




#!/usr/bin/env python3


"""
A TestRunner for use with the Python unit testing framework. It
generates a HTML report to show the result at a glance.

The simplest way to use this is to invoke its main method. E.g.

    import unittest
    import HTMLTestRunner


    ... define your tests ...

    if __name__ == '__main__':
        HTMLTestRunner.main()


For more customization options, instantiates a HTMLTestRunner object.
HTMLTestRunner is a counterpart to unittest's TextTestRunner. E.g.

    # output to a file
    fp = file('my_report.html', 'wb')
    runner = HTMLTestRunner.HTMLTestRunner(
                stream=fp,
                title='My unit test',
                description='This demonstrates the report output by HTMLTestRunner.'
                )

    # Use an external stylesheet.
    # See the Template_mixin class for more customizable options
    runner.STYLESHEET_TMPL = '<link rel="stylesheet" href="my_stylesheet.css" type="text/css">'

    # run the test
    runner.run(my_test_suite)


"""

from Octopus.OctopusHTMLReport.OctopusTestCase import TestCase
from xml.sax import saxutils
import sys
import datetime
from Octopus.OctopusHTMLReport.OctopusTestResult import TestResult
from Octopus.OctopusHTMLReport.OctopusTestCase import TestCase

__author__ = "Abdelghany Elsayed"
__update__ = "Octopus"
__version__ = "1.0"


"""
Change History
Version 1.0
* Use extent report HTML templete create test report. 
"""

# ----------------------------------------------------------------------
# Template


class Template_mixin(object):
    """
    Define a HTML template for report customerization and generation.

    Overall structure of an HTML report

    HTML
    +------------------------+
    |<html>                  |
    |  <head>                |
    |                        |
    |   STYLESHEET           |
    |   +----------------+   |
    |   |                |   |
    |   +----------------+   |
    |                        |
    |  </head>               |
    |                        |
    |  <body>                |
    |                        |
    |   HEADING              |
    |   +----------------+   |
    |   |                |   |
    |   +----------------+   |
    |                        |
    |   REPORT               |
    |   +----------------+   |
    |   |                |   |
    |   +----------------+   |
    |                        |
    |   ENDING               |
    |   +----------------+   |
    |   |                |   |
    |   +----------------+   |
    |                        |
    |  </body>               |
    |</html>                 |
    +------------------------+
    """

    STATUS = {
        0: 'pass',
        1: 'fail',
        2: 'error',
        3: 'info',
    }

    DEFAULT_TITLE = 'OctopusHTMLTestReport'
    DEFAULT_DESCRIPTION = 'OctopusHTMLTestReport'

    # ------------------------------------------------------------------------
    # HTML Template

    HTML_TMPL = r"""
    <!DOCTYPE html>
    <html>
    <head>
        <meta charset='utf-8'/>
        <meta name='description' content=''/>
        <meta name='robots' content='noodp, noydir'/>
        <meta name='viewport' content='width=device-width, initial-scale=1'/>
        <meta id="timeStampFormat" name="timeStampFormat" content='MMM d, yyyy hh:mm:ss a'/>

        <link href='https://fonts.googleapis.com/css?family=Source+Sans+Pro:400,600' rel='stylesheet' type='text/css'>
        <link href="https://fonts.googleapis.com/icon?family=Material+Icons" rel="stylesheet">

        <link href='https://the-octopus.github.io/OctopusHTMLReportResources/extent.css' type='text/css' rel='stylesheet'/>

        <title>%(title)s - TestReport</title>
        %(stylesheet)s        
    </head>

    <body class='extent standard default hide-overflow'>
        <div id='theme-selector' alt='Switch theme, default black' title='Switch theme'>
            <span><i class='material-icons'>desktop_windows</i></span>
        </div>

        %(heading)s

        <div class='container'>
        %(dashboard_view)s
        %(report)s
            
        </div>        
    </body>
    %(script_js)s
    </html>
    """
    # variables: (title, generator, stylesheet, heading, heading, ending)

    # ------------------------------------------------------------------------
    # Stylesheet
    #
    # alternatively use a <link> for external style sheet, e.g.
    #   <link rel="stylesheet" href="$url" type="text/css">

    NAV = """
    <nav>
    <div class="nav-wrapper">
        <a href="#!" class="brand-logo black">
        <img src="https://the-octopus.github.io/OctopusHTMLReportResources/Logo.png"/>
        </a>

        <!-- slideout menu -->
        <ul id='slide-out' class='side-nav fixed hide-on-med-and-down'>
            <li class='waves-effect active'><a href='#!' view='test-view'
                                               onclick="configureView(0);chartsView('test');"><i class='material-icons'>dashboard</i></a>
            </li>
            <!-- <li class='waves-effect'><a href='#!' view='category-view' onclick="configureView(1)"><i
                    class='material-icons'>label_outline</i></a></li>-->
            <li class='waves-effect'><a href='#!' onclick="configureView(-1);chartsView('dashboard');"
                                        view='dashboard-view'><i class='material-icons'>track_changes</i></a></li> 
        </ul>

        <!-- report name -->
        <span class='report-name'>Test Report: %(title)s</span>

        <!-- report headline -->
        <span class='report-headline'></span>

        <!-- nav-right -->
        <ul id='nav-mobile' class='right hide-on-med-and-down nav-right'>
            <li>
                <a href='#!'>
                    <span class='label suite-start-time blue darken-3'>Start Time: %(start_time)s</span>
                </a>
            </li>
            <li>
                <a href='#!'>
                    <span class='label blue darken-3'>Duration: %(duration)s </span>
                </a>
            </li>
        </ul>
    </div>
</nav>
    """

    TEST_VIEW = """
    <div id='test-view' class='view'>
        %(control_section)s
        %(view_charts)s
        %(test_list)s
        
        <div class='subview-right left'>
            <div class='view-summary'>
            
                <div id="step-filters" class="right sr-filters">
                    <a class="btn-floating waves-effect waves-light green" status="pass" alt="pass" title="pass"><i class="material-icons">check_circle</i></a>
                    <a class="btn-floating waves-effect waves-light red" status="fail" alt="fail" title="fail"><i class="material-icons">cancel</i></a>
                    <a class="btn-floating waves-effect waves-light red darken-4" status="fatal" alt="fatal" title="fatal"><i class="material-icons">cancel</i></a>
                    <a class="btn-floating waves-effect waves-light pink text-lighten-1" status="error" alt="error" title="error"><i class="material-icons">error</i></a>
                    <a class="btn-floating waves-effect waves-light orange" alt="warning" status="warning" title="warning"><i class="material-icons">warning</i></a>
                    <a class="btn-floating waves-effect waves-light teal" status="skip" alt="skip" title="skip"><i class="material-icons">redo</i></a>
                    <a class="btn-floating waves-effect waves-light grey" status="clear" alt="Clear filters" title="Clear filters"><i class="material-icons">clear</i></a>
                </div>

                <h5 class='test-name'>%(test_case_name)s</h5>


            </div>
        </div>
    </div>
    %(category_view)s
    
"""
    CONTROL_SECTION = """
    <section id='controls'>
            <div class='controls grey lighten-4'>
                <!-- test toggle -->
                <div class='chip transparent'>
                    <a class='dropdown-button tests-toggle' data-activates='tests-toggle' data-constrainwidth='true'
                       data-beloworigin='true' data-hover='true' href='#'>
                        <i class='material-icons'>warning</i> Status
                    </a>
                    <ul id='tests-toggle' class='dropdown-content'>
                        <li status='pass'><a href='#!'>Pass <i class='material-icons green-text'>check_circle</i></a></li>
                        <li status='fail'><a href='#!'>Fail <i class='material-icons red-text'>cancel</i></a></li>
                        <li status='error'><a href='#!'>Error <i class='material-icons orange-text'>error</i></a></li>
                        <li status="skip"><a href="#!">Skip <i class="material-icons cyan-text">redo</i></a></li>
                        <li class='divider'></li>
                        <li status='clear' clear='true'><a href='#!'>Clear Filters <i class='material-icons'>clear</i></a></li>
                    </ul>
                </div>
                <!-- test toggle -->

                <!-- category toggle -->
                <!--<div class='chip transparent'>
                    <a class='dropdown-button category-toggle' data-activates='category-toggle'
                       data-constrainwidth='false' data-beloworigin='true' data-hover='true' href='#'>
                        <i class='material-icons'>local_offer</i> Category
                    </a>
                    <ul id='category-toggle' class='dropdown-content'>
                        %(suite_name)s
                        <li class='divider'></li>
                        <li class='clear'><a href='#!' clear='true'>Clear Filters</a></li>
                    </ul>
                </div> -->
                <!-- category toggle -->

                <!-- clear filters -->
                <div class='chip transparent hide'>
                    <a class='' id='clear-filters' alt='Clear Filters' title='Clear Filters'>
                        <i class='material-icons'>close</i> Clear
                    </a>
                </div>
                <!-- clear filters -->

                <!-- enable dashboard -->
                <div id='toggle-test-view-charts' class='chip transparent'>
                    <a class='pink-text' id='enable-dashboard' alt='Enable Dashboard' title='Enable Dashboard'>
                        <i class='material-icons'>track_changes</i> Dashboard
                    </a>
                </div>
                <!-- enable dashboard -->

                <!-- search -->
                <div class='chip transparent' alt='Search Tests' title='Search Tests'>
                    <a href="#" class='search-div'>
                        <i class='material-icons'>search</i> Search
                    </a>

                    <div class='input-field left hide'>
                        <input style="color: red" id='search-tests' type='text' class='validate browser-default'
                               placeholder='Search Tests...'>
                    </div>

                </div>
                <!-- search -->
            </div>
        </section>
    """
    SECTION_SUIT_NAME = """
    <li><a href='#'>%(desc)s</a></li>
    """

    VIEW_CHARTS = """
    <div id='test-view-charts' class='subview-full'>
            <div id='test-view-charts' class='subview-full'>
                <div id='charts-row' class='row nm-v nm-h'>
                    <div class='col s12 m6 l6 np-h'>
                        <div class='card-panel nm-v'>
                            <div class='left panel-name'>Tests</div>
                            <div class='chart-box' style='max-height:94px;'>
                                <canvas id='parent-analysis' width='884' height='94' style='display: block; width: 884px; height: 94px;' class='chartjs-render-monitor'></canvas>
                            </div>
                            <div class='block text-small'>
                                <span class='tooltipped' data-position='top' data-tooltip='50%%'><span class='strong'>%(pass_count)s</span> test(s) passed, </span>
                                    <span class='tooltipped' data-position='top' data-tooltip='50%%'><span class='strong'>%(fail_count)s</span> test(s) failed, </span>
                            </div>
                            <div class='block text-small'>
                                <span class='strong tooltipped' data-position='top' data-tooltip='50%%'>%(error_count)s</span> test(s) others
                           </div>
                        </div>
                    </div>
                    <div class='col s12 m6 l6 np-h'>
                        <div class='card-panel nm-v'>
                            <div class='left panel-name'>Steps</div>
                            <div class='chart-box' style='max-height:94px;'>
                                <canvas id='child-analysis' width='884' height='94' class='chartjs-render-monitor' style='display: block; width: 884px; height: 94px;'></canvas>
                            </div>
                             <div class='block text-small'>
                                <span class='tooltipped' data-position='top' data-tooltip='50%%'><span class='strong'>%(steps_pass_count)s</span> step(s) passed, </span>
                                    <span class='tooltipped' data-position='top' data-tooltip='50%%'><span class='strong'>%(steps_fail_count)s</span> step(s) failed, </span>
                            </div>
                            <div class='block text-small'>
                                <span class='strong tooltipped' data-position='top' data-tooltip='50%%'>%(steps_error_count)s</span> step(s) others
                           </div>
                        </div>
                    </div>
                </div>
            </div>
    </div>
    """

    SUBVIEW_LEFT = """
    <div class="subview-left left" style="resize: horizontal; height: 229px;">
		<div class="view-summary" style="height: 211px;">
                <h5>Test Cases</h5>
                <ul id='test-collection' class='test-collection'>
                %(test_collection)s
                </ul>
            </div>
    </div>            
    """

    TEST_COLLECTION = """
    %(li_test_active)s
    <div class='test-heading'>
        <span class='test-name'>%(desc)s</span>
        <span class='test-time'>desc: %(doc)s </span>
        %(status_span)s
    </div>
    <div class='test-content hide'>
        <div class='test-desc'>Pass: %(Pass)s ;
                                Fail: %(fail)s ;
                                Error: %(error)s ;
        </div>
        <div class='test-attributes'>
            <div class='category-list'>
                <span class='category label white-text'>%(desc)s</span>
            </div>
        </div>
        <ul class='collapsible node-list' data-collapsible='accordion'>            
            <!-- (tesfft_collection_ul_list)s -->
             %(node_level)s
                <div class='collapsible-header'>
                    <div class='node-name'>%(desc)s</div>
                    <span class='node-time'>desc: %(doc)s</span>
                    %(status_span)s
                </div>
                <div class='collapsible-body'>
                    <div class='category-list right'>
                        <span class='category label white-text'>%(desc)s</span>
                    </div>
                    <div class='node-steps'>
                        <table class='bordered table-results'>
                            <thead>
                            <tr>
                                <th>Id</th>
                                <th>Status</th>
                                <th>Timestamp</th>
                                <th>Details</th>
                            </tr>
                            </thead>
                            <tbody>
                            %(t_body)s
                            </tbody>
                        </table>
                    </div>
                </div>
            </li>
        </ul>
    </div>
    """
    TEST_COLLECTION_UL_LIST = """
            %(node_level)s
                <div class='collapsible-header'>
                    <div class='node-name'>%(desc)s</div>
                    <span class='node-time'>desc: %(doc)s</span>
                    %(status_span)s
                </div>
                <div class='collapsible-body'>
                    <div class='category-list right'>
                        <span class='category label white-text'>%(desc)s</span>
                    </div>
                    <div class='node-steps'>
                        <table class='bordered table-results'>
                            <thead>
                            <tr>
                                <th>Id</th>
                                <th>Status</th>
                                <th>Timestamp</th>
                                <th>Details</th>
                            </tr>
                            </thead>
                            <tbody>
                            %(t_body)s
                            </tbody>
                        </table>
                    </div>
                </div>
            </li>
    """
    TBODY = """
        <tr class='log info' status='info'>
            <td class='timestamp'>%(step_id)s</td>
            <td class='status info' title='info' alt='info'><i class='material-icons'>info_outline</i></td>
            <td class='timestamp'>%(steptime)s</td>
            <td class='step-details'>
                %(stepcomment)s
                %(image)s  
            </td>
        </tr>        
    """
    TBODY_FAIL = """
        <tr class='log fail' status='fail'>
            <td class='timestamp'>%(step_id)s</td>
            <td class='status fail' title='fail' alt='fail'><i class='material-icons'>cancel</i></td>
            <td class='timestamp'>%(steptime)s</td>
            <td class='step-details'> 
                %(stepcomment)s
                %(image)s                
            </td>
		</tr>
    """

    TBODY_ERROR = """
        <tr class='log error' status='error'>
            <td class='timestamp'>%(step_id)s</td>
            <td class='status fail' title='fail' alt='fail'><i class='material-icons'>error</i></td>
            <td class='timestamp'>%(steptime)s</td>
            <td class='step-details' > 
                %(stepcomment)s
                %(image)s                
            </td>
		</tr>
    """

    TBODY_PASS = """
        <tr class='log pass' status='pass'>
            <td class='timestamp'>%(step_id)s</td>
            <td class='status pass' title='pass' alt='pass'><i class='material-icons'>check_circle</i></td>
            <td class='timestamp'>%(steptime)s</td>
            <td class='step-details' >
                %(stepcomment)s
                %(image)s                
            </td>
        </tr>
    """
    CATEGORY_VIEW = """
    <div id='category-view' class='view hide'>
        <section id='controls'>
            <div class='controls grey lighten-4'>
                <!-- search -->
                <div class='chip transparent' alt='Search Tests' title='Search Tests'>
                    <a href="#" class='search-div'>
                        <i class='material-icons'>search</i> Search
                    </a>
    
                    <div class='input-field left hide'>
                        <input tyle="color: red;" id='search-tests' type='text'
                               class='validate browser-default'
                               placeholder='Search Tests...'>
                    </div>
    
                </div>
                <!-- search -->
            </div>
        </section>
        
        <div class='subview-left left'>

            <div class='view-summary'>
                <h5>Categories</h5>
                <ul id='category-collection' class='category-collection'>

                    <li class='category displayed active'>
                        <div class='category-heading'>
                            <span class='category-name'>Scenarios</span>
                            <span class='category-status right'>
                                <span class='label pass'>%(Pass)s </span>
                                <span class='label fail'>%(fail)s</span>
                            </span>
                        </div>
                        <div class='category-content hide'>
                            <div class='category-status-counts'>
                                <span class='label green accent-4 white-text'>Passed: %(Pass)s</span>
                                <span class='label red lighten-1 white-text'>Failed: %(fail)s</span>
                                <span class='label blue lighten-1 white-text'>Errored: %(error)s</span>
                                <span class="label yellow darken-2 white-text">Skipped: </span>
                            </div>

                            <div class='category-tests'>
                                <table class='bordered table-results'>
                                    <thead>
                                    <tr>
                                        <th>Timestamp</th>
                                        <th>TestName</th>
                                        <th>Status</th>
                                    </tr>
                                    </thead>
                                    <tbody>
                                    %(category_tbody)s
                                    </tbody>
                                </table>
                            </div>
                        </div>
                    </li>
                    %(category_active)s
                <div class='subview-right left'>
            <div class='view-summary'>
                <h5 class='category-name'></h5>
            </div>
        </div>
    </div>
                    
        
    """
    CATEGORY_TBODY = """
    <tr style="border: 1px solid #49cc90; background-color: rgba(73, 204, 144, .1)">
        <td>%(start_time)s</td>
        <td class='linked' test-id='%(name)s_%(cid)s'>%(desc)s</td>
        <td>%(category_tbody_td)s</td>
    </tr>
    <tr>
        <td></td>
        <td class='linked' test-id='暂未处理'></td>
        <td>%(category_tbody_td)s</td>
    </tr>
        
    """

    CATEGORY_ACTIVE = """
    <li class='category displayed active'>
        <div class='category-heading'>
            <span class='category-name'>%(desc)s</span>
            <span class='category-status right'>
                <span class='label pass'>%(Pass)s </span>
                <span class='label fail'>%(fail)s</span>
            </span>
        </div>
        <div class='category-content hide'>
            <div class='category-status-counts'>
                <span class='label green accent-4 white-text'>Passed: %(Pass)s</span>
                <span class='label red lighten-1 white-text'>Failed: %(fail)s</span>
                <span class='label blue lighten-1 white-text'>Errored: %(error)s</span>
            </div>

            <div class='category-tests'>
                <table class='bordered table-results'>
                    <thead>
                    <tr>
                        <th>Timestamp</th>
                        <th>TestName</th>
                        <th>Status</th>
                    </tr>
                    </thead>
                    <tbody>
                    
                    </tbody>
                </table>
            </div>
        </div>
    </li>
    """
    DASHBOARD_VIEW = """
    <div id='dashboard-view' class='view hide'>
        <div class='card-panel transparent np-v'>
            <h5>Dashboard</h5>

            <div class='row'>
                <div class='col s2'>
                    <div class='card-panel r'>
                        Pass
                        <div class='panel-lead'>%(Pass)s</div>
                    </div>
                </div>
                <div class='col s2'>
                    <div class='card-panel r'>
                        Fail
                        <div class='panel-lead'>%(fail)s</div>
                    </div>
                </div>
                <div class='col s2'>
                    <div class='card-panel r'>
                        Error
                        <div class='panel-lead'>%(error)s</div>
                    </div>
                </div>
                <div class='col s2'>
                    <div class='card-panel r'>
                        Skip
                        <div class='panel-lead'></div>
                    </div>
                </div>
                <div class='col s2'>
                    <div class='card-panel r'>
                        Start
                        <div class='panel-lead'>%(start_time)s</div>
                    </div>
                </div>
                <div class='col s2'>
                    <div class='card-panel r'>
                        Time Taken
                        <div class='panel-lead'>%(duration)s seconds</div>
                    </div>
                </div>
                <div class='col s4'>
                    <div class='card-panel'>
                        <span class='right label cyan white-text'>Categories</span>
                        <p>&nbsp;</p>
                        <table>
                            <tr>
                                <th>Name</th>
                                <th>Passed</th>
                                <th>Failed</th>
                                <th>Errored</th>
                                <th>Skipped</th>
                            </tr>
                            <tr>
                                <td>Scenarios</td>
                                <td class="pass">%(Pass)s</td>
                                <td class="fail">%(fail)s</td>
                                <td class="error">%(error)s</td>
                                <td class="skip"></td>
                            </tr>

                                <tr>
                                    <td id="unknown"></td>
                                    <td class="pass"></td>
                                    <td class="fail"></td>
                                    <td class="error"></td>
                                    <td class="skip"></td>
                                </tr>
                        </table>
                    </div>
                </div>
            </div>
        </div>
    </div>
    """

    SCRIPT_JS = """
    <script>
        var test_suite_success = %(Pass)s;
        
    
        var statusGroup = {
            passParent: %(Pass)s,
            failParent: %(fail)s,
            fatalParent: 0,
            errorParent: %(error)s,
            warningParent: 0,
            skipParent: 0,
            exceptionsParent: 0,
    
            passChild: %(steps_pass)s,
            failChild: %(steps_fail)s,
            fatalChild: 0,
            errorChild: %(steps_error)s,
            warningChild: 0,
            skipChild: 0,
            infoChild: 0,
            exceptionsChild: 0,

            passGrandChild: 0,
            failGrandChild: 0,
            fatalGrandChild: 0,
            errorGrandChild: 0,
            warningGrandChild: 0,
            skipGrandChild: 0,
            infoGrandChild: 0,
            exceptionsGrandChild: 0,
        };
    
    </script>
    
    <script src='https://the-octopus.github.io/OctopusHTMLReportResources/extent.js' type='text/javascript'></script>
    
    
    <script type='text/javascript'>
        $(window).off("keydown");
    </script>
    """
    STYLESHEET_TMPL = """
    <style type="text/css">
        .node.level-1 ul {
            display: none;
        }

        .node.level-1.active ul {
            display: block;
        }

        .card-panel.environment th:first-child {
            width: 30%;
        }
        .small_img{
            height: 180px; 
            width: 100px; 
            padding: 10px;
            float: left;
            background-repeat: no-repeat; 
            background-position: center center; 
            background-size: cover; 
          } 
        .black_overlay{ 
            display: none; 
            position: absolute; 
            top: 0%; 
            left: 0%; 
            width: 100%; 
            height: 100%; 
            background-color: white; 
            z-index:1001; 
            -moz-opacity: 0.8; 
            opacity:.80; 
            filter: alpha(opacity=80);  
        } 
        .big_img { 
            cursor: pointer;
            display: none; 
            position: absolute; 
            height: 650px;
            left:50%; 
            top: 50%;
            margin: -300px 0px 0px -200px;
            z-index:1002; 
            overflow: auto; 
        }
    </style>
"""

    # ------------------------------------------------------------------------
    # Heading
    #

    HEADING_ATTRIBUTE_TMPL = """
    Remove
    <p class='attribute'><strong>%(name)s:</strong> %(value)s</p>
"""  # variables: (name, value)

    # ------------------------------------------------------------------------
    # Report
    #

    REPORT_TMPL = """
        <p id='show_detail_line'>Show
        <a href='javascript:showCase(0)'>Summary</a>
        <a href='javascript:showCase(1)'>Failed</a>
        <a href='javascript:showCase(2)'>All</a>
        </p>
        <table id='result_table'>
        <colgroup>
        <col align='left' />
        <col align='right' />
        <col align='right' />
        <col align='right' />
        <col align='right' />
        <col align='right' />
        </colgroup>
        <tr id='header_row'>
            <td>Test Group/Test case</td>
            <td>Count</td>
            <td>Pass</td>
            <td>Fail</td>
            <td>Error</td>
            <td>View</td>
            <td>Screenshot</td>
        </tr>
        %(test_list)s
        <tr id='total_row'>
            <td>Total</td>
            <td>%(count)s</td>
            <td>%(Pass)s</td>
            <td>%(fail)s</td>
            <td>%(error)s</td>
            <td>&nbsp;</td>
            <td>&nbsp;</td>

        </tr>
        </table>
    """  # variables: (test_list, count, Pass, fail, error)

    REPORT_CLASS_TMPL = r"""
        <tr class='%(style)s'>
            <td>%(desc)s</td>
            <td>%(count)s</td>
            <td>%(Pass)s</td>
            <td>%(fail)s</td>
            <td>%(error)s</td>
            <td><a href="javascript:showClassDetail('%(cid)s',%(count)s)">Detail</a></td>
            <td>&nbsp;</td>

        </tr>
    """  # variables: (style, desc, count, Pass, fail, error, cid)

    REPORT_TEST_WITH_OUTPUT_TMPL = r"""
        <tr id='%(tid)s' class='%(Class)s'>
            <td class='%(style)s'><div class='testcase'>%(desc)s</div></td>
            <td colspan='5' align='center'>

            <!--css div popup start-->
            <a class="popup_link" onfocus='this.blur();' href="javascript:showTestDetail('div_%(tid)s')" >
                %(status)s</a>

            <div id='div_%(tid)s' class="popup_window">
                <div style='text-align: right; color:red;cursor:pointer'>
                <a onfocus='this.blur();' onclick="document.getElementById('div_%(tid)s').style.display = 'none' " >
                [x]</a>
                </div>
                <pre>
                %(script)s
                </pre>
            </div>
            <!--css div popup end-->

            </td>
            <td align='center'>
            %(images)s
            </td>
        </tr>
    """  # variables: (tid, Class, style, desc, status)

    REPORT_IMAGE = r"""    
    <img class="r-img" onerror="this.style.display=&quot;none&quot;" data-featherlight="%(screenshot)s" src="%(screenshot)s"  onclick="document.getElementById('light_%(screenshot_id)s').style.display ='block';document.getElementById('fade_%(screenshot_id)s').style.display='block'"/>
    """
    # <img class="r-img" onerror="this.style.display=none;" data-featherlight="%(screenshot)s" src="%(screenshot)s" data-src="%(screenshot)s">

    REPORT_TEST_NO_OUTPUT_TMPL = r"""
        <tr id='%(tid)s' class='%(Class)s'>
            <td class='%(style)s'><div class='testcase'>%(desc)s</div></td>
            <td colspan='5' align='center'>%(status)s</td>
        </tr>
    """  # variables: (tid, Class, style, desc, status)

    REPORT_TEST_OUTPUT_TMPL = r"""
        %(output)s
    """  # variables: (id, output)

    REPORT_TEST_OUTPUT_IMAGE = r""" 
        %(screenshot)s
    """
    REPORT_TEST_OUTPUT_CASEID = r"""
        %(case_id)s
    """
    # ------------------------------------------------------------------------
    # ENDING
    #

    ENDING_TMPL = """<div id='ending'>&nbsp;</div>"""
# -------------------- The end of the Template class -------------------


class HTMLTestReport(Template_mixin):
    """
    This is the class to generate the report from the HTML template
    and write the data to the file
    """

    def __init__(self, stream=sys.stdout, title=None, description=None):
        # all stesp for all test cases
        self.nps = 0
        self.nfs = 0
        self.nes = 0
        self.stream = stream
        if title is None:
            self.title = self.DEFAULT_TITLE
        else:
            self.title = title
        if description is None:
            self.description = self.DEFAULT_DESCRIPTION
        else:
            self.description = description

        self.startTime = datetime.datetime.now()


    def sortResult(self, result_list):
        # Here at least we want to group them together by method or class.
        rmap = {}
        classes = []
        for n, t in result_list:
            cls = t.className
            # if not rmap.has_key(cls):
            if not cls in rmap:
                rmap[cls] = []
                classes.append(cls)
            rmap[cls].append((n, t))
        r = [(cls, rmap[cls]) for cls in classes]
        return r

    def getReportAttributes(self, result):
        """
        Return report attributes as a list of (name, value).
        Override this to add custom attributes.
        """
        startTime = str(self.startTime)[:19]
        duration = str((self.stopTime - self.startTime) +
                       datetime.timedelta(0, 1)).split('.')[0]
        status = []
        if result.success_count:
            status.append('Pass %s' % result.success_count)
        if result.failure_count:
            status.append('Failure %s' % result.failure_count)
        if result.error_count:
            status.append('Error %s' % result.error_count)
        if status:
            status = ' '.join(status)
        else:
            status = 'none'
        return [
            ('Start Time', startTime),
            ('Duration', duration),
            ('Status', status),
        ]

    def generateReport(self, result):
        self.stopTime = datetime.datetime.now()
        report_attrs = self.getReportAttributes(result)
        generator = 'OctopusHTMLTestReport %s' % __version__
        stylesheet = self._generate_stylesheet()
        heading = self._generate_heading(report_attrs)
        report = self._generate_report(result)
        ending = self._generate_ending()
        dashboard_view = self._generate_dashboardView(report_attrs, result)
        scriptJS = self._generate_script(result)

        output = self.HTML_TMPL % dict(
            title=saxutils.escape(self.title),
            generator=generator,
            stylesheet=stylesheet,
            heading=heading,
            report=report,
            ending=ending,
            dashboard_view=dashboard_view,
            script_js=scriptJS,
        )
        self.stream.write(output.encode('utf8'))

    def _generate_stylesheet(self):
        return self.STYLESHEET_TMPL

    def _generate_heading(self, report_attrs):
        # Discard HEADING_ATTRIBUTE_TMPL
        a_lines = []
        startTime = ""
        duration = ""
        for name, value in report_attrs:
            line = self.HEADING_ATTRIBUTE_TMPL % dict(
                name=saxutils.escape(name),
                value=saxutils.escape(value),
            )
            a_lines.append(line)
            startTime = value if name == "Start Time" else startTime
            duration = value if name == "Duration" else duration
        # take out Start Time、Duration、Status
        heading = self.NAV % dict(
            title=saxutils.escape(self.title),
            start_time=startTime,
            duration=duration,
            description=saxutils.escape(self.description),
        )
        return heading

    def _generate_script(self, result):
        scriptJS = self.SCRIPT_JS % dict(
            Pass=str(result.success_count),
            fail=str(result.failure_count),
            error=str(result.error_count),
            steps_pass=str(self.nps),
            steps_fail=str(self.nfs),
            steps_error=str(self.nes),

        )
        return scriptJS

    def _generate_dashboardView(self, report_attrs, result):
        a_lines = []
        startTime = ""
        duration = ""
        for name, value in report_attrs:
            line = self.HEADING_ATTRIBUTE_TMPL % dict(
                name=saxutils.escape(name),
                value=saxutils.escape(value),
            )
            a_lines.append(line)
            startTime = value if name == "Start Time" else startTime
            duration = value if name == "Duration" else duration
        dashboard_view = self.DASHBOARD_VIEW % dict(
            Pass=str(result.success_count),
            fail=str(result.failure_count),
            error=str(result.error_count),
            start_time=startTime,
            duration=duration,
        )
        return dashboard_view

    def _generate_report(self, result):
        rows = []
        row1s = []
        section_name = []
        categoryTbody = []
        categoryActive = []
        testCases = result.result

        for cid, (case_status, case_result) in enumerate(testCases):
            # (methodName,description ,success_count,failure_count ,error_count ,steps)

            desc = name = doc = case_result.methodName+'_'+str(cid+1)

            #  Test cases statistics
            np = case_result.success_count
            nf = case_result.failure_count
            ne = case_result.error_count

            # section in suite name
            sectionName = self.SECTION_SUIT_NAME % dict(
                desc=desc,
            )
            section_name.append(sectionName)

            # generate the details view rows
            testCollectionUlList = []

            for tid, (code, time, details, sceenchoot) in enumerate(case_result.steps):
                if code == 0:
                    self.nps += 1
                elif code == 1:
                    self.nfs += 1
                elif code == 2:
                    self.nes += 1
                else:
                    self.nps += 1
                self._generate_report_test(
                    rows, cid, tid, code, time, details, sceenchoot, case_result, testCollectionUlList)

            if ne > 0:
                liTestActive = '<li class="test displayed active has-leaf error" status="error" bdd="error" test-id="' + \
                    desc + '_' + str(cid+1) + '">'
                statusSpan = '<span class="test-status right error">error</span>'
                nodeLevel = '<li class="node level-1 leaf error" status="error" test-id="' + \
                    desc + '_' + str(cid + 1) + '">'
                categoryTbodyTd = '<span class="test-status error">error</span>'
            elif nf > 0:
                liTestActive = '<li class="test displayed active has-leaf fail" status="fail" bdd="false" test-id="' + \
                    desc + '_' + str(cid + 1) + '">'
                statusSpan = '<span class="test-status right fail">fail</span>'
                nodeLevel = '<li class="node level-1 leaf fail" status="fail" test-id="' + \
                    desc + '_' + str(cid + 1) + '">'
                categoryTbodyTd = '<span class="test-status fail">fail</span>'
            else:
                liTestActive = '<li class="test displayed active has-leaf pass" status="pass" bdd="true" test-id="' + \
                    desc + '_' + str(cid+1) + '">'
                statusSpan = '<span class="test-status right pass">pass</span>'
                nodeLevel = '<li class="node level-1 leaf pass" status="pass" test-id="' + \
                    desc + '_' + str(cid + 1) + '">'
                categoryTbodyTd = '<span class="test-status pass">pass</span>'

            row1 = self.TEST_COLLECTION % dict(
                li_test_active=liTestActive,
                status_span=statusSpan,
                node_level=nodeLevel,
                desc=desc,
                doc=doc,
                count=np+nf+ne,
                Pass=np,
                fail=nf,
                error=ne,
                # test_collection_ul_list=''.join(testCollectionUlList),
                t_body=''.join(testCollectionUlList),
                cid='c%s' % (cid+1),
            )
            row1s.append(row1)

            category_tbody = self.CATEGORY_TBODY % dict(
                name=name,
                desc=desc,
                start_time=self.startTime,
                cid=cid,
                category_tbody_td=categoryTbodyTd,
            )
            categoryTbody.append(category_tbody)

            category_active = self.CATEGORY_ACTIVE % dict(
                desc=desc,
                Pass=np,
                fail=nf,
                error=ne,
            )
            categoryActive.append(category_active)

        controlSection = self.CONTROL_SECTION % dict(
            suite_name=''.join(section_name)
        )

        viewCharts = self.VIEW_CHARTS % dict(
            pass_count=str(result.success_count),
            fail_count=str(result.failure_count),
            error_count=str(result.error_count),

            steps_pass_count=str(self.nps),
            steps_fail_count=str(self.nfs),
            steps_error_count=str(self.nes),
        )
        subviewLeft = self.SUBVIEW_LEFT % dict(
            test_collection=''.join(row1s),
        )
        category_view = self.CATEGORY_VIEW % dict(
            Pass=str(result.success_count),
            fail=str(result.failure_count),
            error=str(result.error_count),
            category_tbody=categoryTbody,
            category_active=categoryActive,
        )

        report = self.TEST_VIEW % dict(
            control_section=controlSection,
            view_charts=viewCharts,
            # test_list=''.join(rows),
            test_list=subviewLeft,
            count=str(result.success_count + \
                      result.failure_count+result.error_count),
            Pass=str(result.success_count),
            fail=str(result.failure_count),
            error=str(result.error_count),
            category_view=category_view,
            test_case_name='test_case_name',
        )
        return report

    def _generate_report_test(self, rows, cid, tid, code, time, details, sceenchoot, case_result, testCollectionUlList):
        # steps is a list of result in 4 tuple
        # (
        #   step type code (0: success; 1: fail; 2: info; 3: error),
        #   timestamp time,
        #   step details string,
        #   sceenchoot string path,
        # )

        # e.g. 'pt1.1', 'ft1.1', etc
        has_output = bool(details)
        title = '%s.%s' % (str(cid+1), str(tid+1))
        tid = (code == 0 and 'p' or 'f') + 't%s.%s' % (cid+1, tid+1)
        name = case_result.id().split('.')[-1] or ""
        doc = case_result.shortDescription() or ""
        desc = doc and ('%s: %s' % (name, doc)) or name

        # doc = name = desc = tid =  't%s.%s' % (cid+1, tid+1)

        tempBODY = ''

        if code == 0:
            tempBODY = tmpl = has_output and self.TBODY_PASS or self.REPORT_TEST_NO_OUTPUT_TMPL
        elif code == 1:
            tempBODY = tmpl = has_output and self.TBODY_FAIL or self.REPORT_TEST_NO_OUTPUT_TMPL
        elif code == 2:
            tempBODY = tmpl = has_output and self.TBODY_ERROR or self.REPORT_TEST_NO_OUTPUT_TMPL
        else:
            tempBODY = tmpl = has_output and self.TBODY or self.REPORT_TEST_NO_OUTPUT_TMPL

        images = ''

        if sceenchoot:
            images = self.REPORT_IMAGE % dict(
                screenshot_id=sceenchoot.split(".")[0],
                screenshot=saxutils.escape(sceenchoot)
            )

        script = self.REPORT_TEST_OUTPUT_TMPL % dict(
            id=cid+1,
            # output=saxutils.escape(uo+ue),
            output=(details.replace("\n", "<br/>")),
        )

        tBody = tempBODY % dict(
            steptime=str(time),
            stepcomment=script,
            image=images,
            step_id=title,
        )

        if code == 1:
            nodeLevel = '<li class="node level-1 leaf fail" status="fail" test-id="' + \
                desc + '_' + str(tid) + '_' + str(cid + 1) + '">'
            statusSpan = '<span class="test-status right fail">fail</span>'
        elif code == 2:
            nodeLevel = '<li class="node level-1 leaf error" status="error" test-id="' + \
                desc + '_' + str(tid) + '_' + str(cid + 1) + '">'
            statusSpan = '<span class="test-status right error">error</span>'
        elif code == 3:
            nodeLevel = '<li class="node level-1 leaf info" status="info" test-id="' + \
                desc + '_' + str(tid) + '_' + str(cid + 1) + '">'
            statusSpan = '<span class="test-status right info">info</span>'
        else:
            nodeLevel = '<li class="node level-1 leaf pass" status="pass" test-id="' + \
                desc + '_' + str(tid) + '_' + str(cid + 1) + '">'
            statusSpan = '<span class="test-status right pass">pass</span>'

        tcll = self.TEST_COLLECTION_UL_LIST % dict(
            node_level=nodeLevel,
            status_span=statusSpan,
            desc=title,
            doc=doc,
            t_body=tBody,
        )

        # testCollectionUlList.append(tcll)
        testCollectionUlList.append(tBody)

        caseid = self.REPORT_TEST_OUTPUT_CASEID % dict(
            case_id=saxutils.escape(details)
        )
        row = tmpl % dict(
            tid=tid,
            Class=(code == 0 and 'hiddenRow' or 'none'),
            style=code == 2 and 'errorCase' or (
                code == 1 and 'failCase' or 'none'),
            desc=desc,
            # script=script,
            # image = image[image.find("image"):(int(image.find("png"))+3)],
            # images=images,
            steptime=str(time),
            stepcomment=script,
            image=images,
            caseid=caseid[caseid.find("case"):(int(caseid.find("case"))+9)],
            status=self.STATUS[code],
            step_id=title,
        )
        rows.append(row)
        if not has_output:
            return

    def _generate_ending(self):
        return self.ENDING_TMPL
